/* Copyright 2008 Daniel Vokó <vokoda@gmail.com>
 * 
 * This file is part of Sagittarius.
 *
 * Sagittarius is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * Sagittarius is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Sagittarius.  If not, see <http://www.gnu.org/licenses/>.
 */


using System;

namespace Sagittarius.Widgets
{
	public partial class DatePicker : Gtk.Bin
	{
		private DateTime _value;
		private int timeZone;
		
		public DatePicker()
		{
			this.Build();
			_value = new DateTime(2000,1,1);
		}
		
		public DateTime Value 
		{
			get
			{ 
				return _value; 
			}
			set
			{ 
				//Console.WriteLine("Entering DatePicker.Date");
				_value=value;
				yearSb.Value=value.Year;
				monthCb.Active = value.Month-1;
				dayCb.Active = value.Day-1;
				hourSb.Value=value.Hour;
				minuteSb.Value=value.Minute;
				secondSb.Value=value.Second;
				//Console.WriteLine("Leaving DatePicker.Date");
			}
		}
		public int TimeZone
		{
			get
			{ return timeZone; }
			set
			{
				if ((value<-12)||(value>12))
				{
					throw new ArgumentOutOfRangeException("TimeZone","The given Time zone value is out of range (-12 - 12)");
				}
				else
				{
					timeZone=value;
					timeZoneSb.Value=timeZone;
				}
			}
		}

		public override string ToString()
		{
			//Console.WriteLine("Entering DatePicker.ToString.");
			string result = yearSb.ValueAsInt.ToString()
				+ " "
				+ monthCb.ActiveText
				+ " "
				+ dayCb.ActiveText;
			//Console.WriteLine("Date: "+result);
			//Console.WriteLine("Exiting DatePicker.ToString.");
			return result;
		}

		protected virtual void OnYearSbValueChanged (object sender, System.EventArgs e)
		{
			//Console.WriteLine("Entering DatePicker.OnYearChanged.");
			//Console.WriteLine("Year: "+yearSb.Value);
			//Console.WriteLine("Month: "+monthCb.Active);
			SetDaysInMonth();
			_value= new DateTime ((int)yearSb.Value,_value.Month,_value.Day,_value.Hour,_value.Minute,_value.Second);
			//Console.WriteLine("The month is: "+ monthCb.Active+1+ " wich is " +(i-1)+" days long");
			//Console.WriteLine("Exiting DatePicker.OnYearChanged.");
		}

		protected virtual void OnMonthCbChanged (object sender, System.EventArgs e)
		{
			//Console.WriteLine("Entering DatePicker.OnMonthChanged.");
			//Console.WriteLine("Year: "+yearSb.Value);
			//Console.WriteLine("Month: "+(monthCb.Active+1));
			SetDaysInMonth();
			_value= new DateTime (_value.Year,(int)monthCb.Active+1,_value.Day,_value.Hour,_value.Minute,_value.Second);
			//Console.WriteLine("The month is: "+ (monthCb.Active+1)+ " wich is " +(i-1)+" days long");
			//Console.WriteLine("Exiting DatePicker.OnMonthChanged.");
		}
		
		private void SetDaysInMonth()
		{
			int oldDay = dayCb.Active;
			//Console.WriteLine("oldday: "+oldDay);
			int daysInMonth = DateTime.DaysInMonth((int)yearSb.Value,monthCb.Active+1);
			Gtk.ListStore dayStore = new Gtk.ListStore(typeof(int));
			int i;
			for(i=1;i<=daysInMonth;i++)
			{
				dayStore.AppendValues(i);
			}
			dayCb.Model=dayStore;
			if (oldDay>=0)
			{
				if(oldDay<=i)
				{ dayCb.Active=oldDay; }
				else
				{ dayCb.Active=i; }
			}
		}

		protected virtual void OnDayCbChanged (object sender, System.EventArgs e)
		{
			//Console.WriteLine("Entering DatePicker.OnDayChange.");
			_value= new DateTime ((int)yearSb.Value,_value.Month,dayCb.Active+1,_value.Hour,_value.Minute,_value.Second);
			//Console.WriteLine("Exiting DatePicker.OnDayChange.");
		}

		protected virtual void OnHourSbValueChanged (object sender, System.EventArgs e)
		{
			//Console.WriteLine("Entering DatePicker.OnHourChange.");
			_value= new DateTime ((int)yearSb.Value,_value.Month,_value.Day,(int)hourSb.Value,_value.Minute,_value.Second);
			//Console.WriteLine("Exiting DatePicker.OnHourChange.");
		}

		protected virtual void OnMinuteSbValueChanged (object sender, System.EventArgs e)
		{
			//Console.WriteLine("Entering DatePicker.OnMinuteChange.");
			_value= new DateTime ((int)yearSb.Value,_value.Month,_value.Day,_value.Hour,(int)minuteSb.Value,_value.Second);
			//Console.WriteLine("Exiting DatePicker.OnMinuteChange.");
		}

		protected virtual void OnSecondSbValueChanged (object sender, System.EventArgs e)
		{
			//Console.WriteLine("Entering DatePicker.OnSecondChange.");
			_value= new DateTime ((int)yearSb.Value,_value.Month,_value.Day,_value.Hour,_value.Minute,(int)secondSb.Value);
			//Console.WriteLine("Exiting DatePicker.OnSecondChange.");
		}

		protected virtual void OnTimeZoneSbValueChanged (object sender, System.EventArgs e)
		{
			//Console.WriteLine("Entering DatePicker.OnTimeZoneChange.");
			timeZone=(int)timeZoneSb.Value;
			//Console.WriteLine("Exiting DatePicker.OnTimeZoneChange.");
		}
		
		public string GetValueAsStandardString()
		{
			string tZ;
			if (timeZone<0)
			{
				tZ= timeZone.ToString()+":000";
			}
			else
			{
				tZ= "+"+timeZone.ToString()+":000";
			}
			if(tZ.Length==6)
			{
				tZ=tZ.Insert(1,"0");
			}
			string year = _value.Year.ToString();
			string month = _value.Month.ToString();
			if(month.Length==1)
			{
				month="0"+month;
			}
			string day = _value.Day.ToString();
			if(day.Length==1)
			{
				day="0"+day;
			}
			string hour = _value.Hour.ToString();
			if(hour.Length==1)
			{
				hour="0"+hour;
			}
			string minute = _value.Minute.ToString();
			if(minute.Length==1)
			{
				minute="0"+minute;
			}
			string second = _value.Second.ToString();
			if(second.Length==1)
			{
				second="0"+second;
			}
			return year+
				":"+
				month+
				":"+
				day+
				" "+
				hour+
				":"+
				minute+
				":"+
				second+
				"."+
				tZ;
		}
		
		public void SetValueFromStandardString(string input)
		{
			int year = Convert.ToInt32(input.Substring(0,4));
			double minimum, maximum;
			yearSb.GetRange(out minimum, out maximum);
			if(year<minimum)
			{
				yearSb.SetRange((double)year-5,maximum);
			}
			Value=new DateTime(
				year,
			    Convert.ToInt32(input.Substring(5,2)),
				Convert.ToInt32(input.Substring(8,2)),
				Convert.ToInt32(input.Substring(11,2)),
				Convert.ToInt32(input.Substring(14,2)),
				Convert.ToInt32(input.Substring(17,2)));
			if(input.Substring(19,1)=="Z")
			{
				timeZone=0;
			}
			else
			{
				TimeZone=Convert.ToInt32(input.Substring(20,3));
			}
		}
	}
}
